<?php

namespace EventPortal\Domain\Field\Models;

use EventPortal\Concerns\Respondable;
use EventPortal\Domain\Field\Enums\FieldType;
use EventPortal\Domain\Response\Models\Response;
use EventPortal\Traits\HasSchemalessAttributes;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOne;
use Spatie\Enum\Laravel\HasEnums;

class Field extends Model
{
    use HasSchemalessAttributes, HasEnums;

    /** @var array<string> */
    protected $casts = [
        'data' => 'array',
    ];

    /** @var array<string> */
    protected $with = [
        'response',
    ];

    /** @var array<string> */
    protected $enums = [
        'type' => FieldType::class,
    ];

    /** @var array<string> */
    protected $guarded = [];

    public function response(): HasOne
    {
        return $this->hasOne(Response::class);
    }

    public function children(): HasMany
    {
        return $this->hasMany(Field::class, 'section_id');
    }

    public function parent(): BelongsTo
    {
        return $this->belongsTo(Field::class, 'section_id');
    }

    public function isChild(): bool
    {
        return $this->section_id !== null;
    }

    public function isParent(): bool
    {
        return $this->section_id === null;
    }

    public function scopeForModel(Builder $query, Respondable $model): Builder
    {
        return $query
            ->where('respondable_id', $model->id)
            ->where('respondable_type', get_class($model));
    }

    public function scopeOnlySections(Builder $query): Builder
    {
        return $query->whereNull('section_id');
    }

    public function scopeOnlyChildren(Builder $query): Builder
    {
        return $query->whereNotNull('section_id');
    }
}
