<?php

namespace EventPortal\Tests\Support\Traits;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Collection;
use PHPUnit\Framework\Assert;

trait InteractsWithModels
{
    /**
     * @param Model $expected
     * @param Model $actual
     * @return static
     */
    public function assertSameModel(Model $expected, $actual)
    {
        Assert::assertTrue(ModelChecker::isSameModel($expected, $actual), 'Failed asserting that two models are the same');

        return $this;
    }

    /**
     * @param Model $expected
     * @param Model $actual
     * @return static
     */
    public function assertNotSameModel(Model $expected, Model $actual)
    {
        Assert::assertFalse(ModelChecker::isSameModel($expected, $actual), 'Failed asserting that two models are not the same');

        return $this;
    }



    /**
     * @param Model $expected
     * @param \Illuminate\Support\Collection|array $collection
     * @return static
     */
    public function assertContainsModel(Model $expected, $collection)
    {
        if (is_array($collection)) {
            $collection = new Collection($collection);
        }

        $collection->assertContainsModel($expected);

        return $this;
    }

    /**
     * @param Model $expected
     * @param \Illuminate\Support\Collection|array $collection
     * @return static
     */
    public function assertNotContainsModel(Model $expected, $collection)
    {
        if (is_array($collection)) {
            $collection = new Collection($collection);
        }

        $collection->assertNotContainsModel($expected);

        return $this;
    }

    /**
     * Asserts the models are the same in value just their ids are different.
     */
    public function assertClonedModel(Model $expected, Model $actual, array $fieldsToExclude = []): self
    {
        $this->assertSame(\get_class($expected), \get_class($actual));
        $this->assertNotEquals($expected->id, $actual->id, 'The cloned id matches the original.');

        $expectedAttributes = array_except(
            $expected->getAttributes(),
            array_merge(['id', 'created_at', 'updated_at', 'deleted_at'], $fieldsToExclude)
        );
        $actualAttributes = array_except(
            $actual->getAttributes(),
            array_merge(['id', 'created_at', 'updated_at', 'deleted_at'], $fieldsToExclude)
        );
        $this->assertEquals($expectedAttributes, $actualAttributes);

        return $this;
    }
}
